/******************************************************************************
 * shadowmap.cpp 
 * implements the construction and drawing of shadow maps
 * Copyright (C) 2010 Marc Gissler <gisslerm@informatik.uni-freiburg.de>
 ******************************************************************************/

#include "shadowmap.h"

#include "shaderProgramDepth.h"
#include "shaderProgramShadowMap.h"

#include "lightmanager.h"
#include "scene.h"
#include "trackball/glTrackball.h"

CShadowMap::CShadowMap() : m_shaderShadowMap(NULL), m_shaderDepth(NULL)
{
	m_shadowMapSize[0] = 64;
	m_shadowMapSize[1] = 64;

  // TODO: Construct FBO
  //check correctness with CShadowMap::m_checkValidity()
}

CShadowMap::~CShadowMap()
{
  // TODO: delete FBO
}

void CShadowMap::build(int lightId, gmVector3& lightPos)
{
	std::map<int, CShadowData*>::iterator mapIter;
	mapIter = m_mapLightAndData.find(lightId);

	if(mapIter == m_mapLightAndData.end())
	{
		CShadowData* data = new CShadowData();
		mapIter = m_mapLightAndData.insert(std::pair<int, CShadowData*>(lightId, data)).first;
	}
}

void CShadowMap::clear()
{
	std::map<int, CShadowData*>::iterator mapIter;
	for(mapIter = m_mapLightAndData.begin(); mapIter != m_mapLightAndData.end(); ++mapIter)
	{
		CShadowData* data = mapIter->second;
		delete data;
	}
	m_mapLightAndData.clear();
}

bool CShadowMap::m_checkValidity()
{
	bool success = true;

	GLuint error = glCheckFramebufferStatusEXT(GL_FRAMEBUFFER);
	switch(error)
	{
	case GL_FRAMEBUFFER_COMPLETE_EXT:
		{
			printf("FrameBufferObjects initialized.");
			break;
		}
	case GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT:
		{
			success = false;
			printf("Incomplete attachment in FBO");
			break;
		}
	case GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT:
		{
			success = false;
			printf("Missing attachment in FBO");
			break;
		}
	case GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT:
		{
			success = false;
			printf("Incomplete dimensions");
			break;
		}
	case GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT:
		{
			success = false;
			printf("Incomplete formats");
			break;
		}
	case GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT:
		{
			success = false;
			printf("Incomplete draw buffer");
			break;
		}
	case GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT:
		{
			success = false;
			printf("Incomplete read buffer");
			break;
		}
	case GL_FRAMEBUFFER_UNSUPPORTED_EXT:
		{
			success = false;
			printf("Framebufferobjects unsupported");
			break;
		}
	}

	return success;
}

void CShadowMap::drawSceneWithShadows()
{
	// for debugging: writes the texture data into an STL vector
	/*int size = m_shadowMapSize[0] * m_shadowMapSize[1] * sizeof(unsigned int);
	unsigned int *bytes = (unsigned int*) malloc( size);
	std::vector<unsigned int> vect;*/

	// 1. pass draw scene as usual

	CScene* pScene = CScene::getScene();
	std::map<int, CShadowData*>::iterator mapIter;

	for(mapIter = m_mapLightAndData.begin(); mapIter != m_mapLightAndData.end(); ++mapIter)
	{
		int lightId = mapIter->first;
		glEnable(GL_LIGHT0+lightId);
	}

	// render the scene with all the lights on
	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);	
	pScene->drawMeshes();
	
	for(mapIter = m_mapLightAndData.begin(); mapIter != m_mapLightAndData.end(); ++mapIter)
	{
		int lightId = mapIter->first;
		glDisable(GL_LIGHT0+lightId);
	}

  // TODO: render to your FBO

  // TODO: render the result as a texture onto a quad

	// for debugging: writes the texture data into an STL vector
	//free(bytes);
}
